from ezgame import Ezgame
from point2d import Point2D
import math
from random import random

ANGLE = 45
NEXT_SIZE = 0.9

class Tree:
    def __init__(self, p, size, level=1, leaf=False):
        self.p = p
        self.size  = size
        self.level = level
        self.right = None
        self.left  = None
        self.leaf  = False


    def create_children(self, levels=1, angle=ANGLE):
        right_point   = Point2D()
        right_point.r = self.size*NEXT_SIZE
        right_point.a = self.p.a - angle
        right_point   = right_point + self.p
        self.right    = Tree(right_point, self.size*NEXT_SIZE, levels)

        left_point   = Point2D()
        left_point.r = self.size*NEXT_SIZE
        left_point.a = self.p.a + angle
        left_point   = left_point + self.p
        self.left    = Tree(left_point, self.size*NEXT_SIZE, levels)


        if levels <= 0:
            self.right.leaf = True
            self.left.leaf  = True
        else:
            self.right.create_children(levels-1, angle)
            self.left.create_children(levels-1, angle)

#############

APPW = 1200
APPH = 800

LEVELS = 9

class Application:
    def __init__(self):
        self.ez = Ezgame(APPW, APPH)
        self.ez.origin = Point2D(APPW/2.0, APPH*0.90)
        # self.ez.fps = FPS
        self.ez.init(self.loop)

        self.size = 1
        p = Point2D(0, 0)
        p.a = math.radians(90)
        self.tree = Tree(p, self.size, LEVELS)
        self.angle = math.radians(1)
        # self.tree.create_children(LEVELS, self.angle)

    def drawNode(self, node):
        if node.level == 1:
            if random() < 0.25:
                self.drawChildren(node, "deeppink2")
            else:
                self.drawChildren(node, "chartreuse2")

        elif node.level == 2:
                self.drawChildren(node, "chartreuse3")
        elif node.level == 3:
                self.drawChildren(node, "chartreuse4")
        else:
                self.drawChildren(node, "burlywood4")

    def drawChildren(self, node, color):
        self.ez.line(node.p, node.right.p, color, node.level)
        self.ez.line(node.p, node.left.p, color, node.level)

        if not node.right.leaf:
            self.drawNode(node.right)
            self.drawNode(node.left)
        return


    def loop(self):
        # self.ez.line(Point2D(), self.tree.p, "burlywood3", 48)
        # print(math.degrees(self.angle*2))
        self.tree.create_children(LEVELS, self.angle)
        self.drawNode(self.tree)

        if self.angle <= math.radians(35):
            self.angle += math.radians(0.1)
        if self.tree.size <= 100:
            self.tree.size += 0.5

    def run(self):
        self.ez.run()


# if __name__ == '__main__':
def run_example():
    app = Application()
    app.run()


import pygame
from pygame.locals import *
from point2d import Point2D

class Ezgame:
    def __init__(self, width=100, height=100):
        self.width  = width
        self.height = height
        self.size   = (self.width, self.height)
        self.origin = Point2D(self.width/2, self.height/2)
        # self.origin = Point2D(300, self.height-150)

        self.screen = None

        self.backgroundColor = '#FFFFFF'
        self.axisColor = '#D0D0D0'
        self.textColor = '#666666'

        self.loopFunction = None
        self.keepRunning = True

        self.clock = pygame.time.Clock()
        self.fps = 60

        self.keys = {
            K_ESCAPE: self.setQuit
        }

        self.gui = True

    def setSize(self, w, h):
        self.width  = w
        self.height = h
        self.size   = (self.width, self.height)
        self.origin = Point2D(self.width/2, self.height/2)


    def transform(self, p):
        # transforms world coordinates to screen coordinates
        return Point2D(p.x+self.origin.x, self.origin.y - p.y)

    def init(self, loopFunction=None):
        pygame.init()
        self.font = pygame.font.SysFont("monospace", 15)
        self.screen = pygame.display.set_mode(self.size)
        # self.screen.fill(Color(self.backgroundColor))
        self.loopFunction = loopFunction

    def line(self, ini, end, color='black', width=1):
        if self.gui:
            pygame.draw.line(
                self.screen,
                Color(color),
                self.transform(ini).ints(),
                self.transform(end).ints(),
                width)

    def lines(self, points, color='black', width=1):
        if self.gui:
            transformed = [self.transform(p).ints() for p in points]
            pygame.draw.lines(
                self.screen,
                Color(color),
                False,
                transformed,
                width)

    def point(self, pos, color='red', size=3):
        if self.gui:
            pygame.draw.circle(
                self.screen, 
                Color(color), 
                self.transform(pos).ints(),
                size
            )

    def text(self, txt, pos, onScreen=False):
        if self.gui:
            label = self.font.render(txt, 1, Color(self.textColor))
            pos_tuple = self.transform(pos).ints()
            if onScreen:
                pos_tuple = pos.ints()
            self.screen.blit(label, pos_tuple)

    def drawScreen(self):
        if self.gui:
            self.screen.fill(Color(self.backgroundColor))
            self.drawAxis()


    def drawAxis(self):
        if self.gui:
            pygame.draw.line(
                self.screen,
                Color(self.axisColor),
                (0, int(self.origin.y)),
                (self.width, int(self.origin.y)),
                1)
            pygame.draw.line(
                self.screen,
                Color(self.axisColor),
                (int(self.origin.x), 0),
                (int(self.origin.x), self.height),
                1)

    def setQuit(self):
        self.keepRunning = False

    def press(self, key, func):
        self.keys[key] = func

    def run(self):
        while self.keepRunning:
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.setQuit()
                elif event.type == KEYDOWN:
                    if event.key in self.keys:
                        self.keys[event.key]()

            if self.gui:
                self.drawScreen()

            if self.loopFunction:
                self.loopFunction()
            pygame.display.flip()

            self.clock.tick(self.fps)






import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ezgame",
    version="0.0.1",
    author="Fabricio J.C. Montenegro",
    author_email="fabriciojcmontenegro@gmail.com",
    description="Small layer over pygame to make its usage more transparent.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SplinterDev/ezgame",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Topic :: Multimedia :: Graphics :: Viewers",
    ],
    install_requires=["pygame", "point2d"]
)